#!perl

#
# Test module which makes sure the YAML->XML->YAML mapping doesn't
# change the underlying information. Loops over the .yaml files in the
# testInputs subdirectory.
#
# Antonio García Domínguez
# $Id: equivalence.t 171 2008-05-03 13:57:56Z bluezio $
#
use strict;
use warnings;

use Test::More;
use XML::LibXML;
use XML::LibXSLT;
use open IN  => ':encoding(UTF-8)';
use open OUT => ':utf8';

use Cwd;
use YAXML::Reverse qw(LoadFile ConvertFile);

# Obtain a list of all test files
my @testFiles = map { m!.*/(.*?)\.yaml!i } <t/testInputs/*.yaml>;

# Notify Test::More about how many tests we're going to run
plan tests => scalar @testFiles;

# Create the LibXML and LibXSLT objects
my $xml  = new XML::LibXML;
my $xslt = new XML::LibXSLT;
my $pathStylesheet   = "share/xml2yaml.xsl";
my $stylesheetSource = $xml->parse_file($pathStylesheet);
my $stylesheet       = $xslt->parse_stylesheet($stylesheetSource);

foreach my $i (@testFiles) {
    my $pathYAML      = "t/testInputs/${i}.yaml";
    my $pathMappedXML = "t/testOutputXML/${i}.xml";
    my $pathRegenYAML = "t/testOutputYAML/${i}.yaml";

    diag("Using $pathYAML...");

    # Original data structure
    my $origYAML = LoadFile($pathYAML);

#    diag("Converting to $pathMappedXML...");

    # Resulting XML text, which we then dump into a file
    my $xmlText  = ConvertFile($pathYAML);

    local *OF;
    open(OF, ">$pathMappedXML")
	or die "$pathMappedXML couldn't be opened for writing: $!";
    print OF $xmlText;
    close(OF);

#    diag("Converting to $pathRegenYAML...");

    # Transform the XML back into YAML using XSLT
    my $source = $xml->parse_file($pathMappedXML);
    my $result = $stylesheet->transform($source);
    my $regenYAML = $stylesheet->output_string($result);

    open(OF, ">$pathRegenYAML")
	or die "$pathRegenYAML couldn't be opened for writing: $!";
    print OF $regenYAML;
    close(OF);

#    diag("Loading $pathRegenYAML...");

    # Load the regenerated YAML and check for equality
    my $finalYAML = LoadFile($pathRegenYAML);

#    diag("Comparing...");

    is_deeply($finalYAML, $origYAML,
	      "Original YAML and regenerated YAML have the same information"); 
}
